#!/bin/bash
# Andre Anjos <andre.dos.anjos@gmail.com>
# Fri Jan 12 09:36:32 2018

# Get latest versions of packages in anaconda 4.4+
# If a package name is specified on the input, just for that package
# Otherwise, the default list of packages is printed

CONDA=/opt/miniconda/bin/conda

PACKAGES=()

if [ $# -gt 0 ]; then
  if [ $1 == '-h' ]; then
    echo "usage: $0  #runs through all supported packages"
    echo "usage: $0 <package> #just queries this package"
    echo "example: $0 numpy"
  else
    PACKAGES+=( "$@" )
  fi
else
  PACKAGES+=('curl')
  PACKAGES+=('cython')
  PACKAGES+=('cytoolz')
  PACKAGES+=('dask')
  PACKAGES+=('freetype')
  PACKAGES+=('hdf5')
  PACKAGES+=('imagesize')
  PACKAGES+=('ipython')
  PACKAGES+=('jpeg')
  PACKAGES+=('libpng')
  PACKAGES+=('libtiff')
  PACKAGES+=('matplotlib')
  PACKAGES+=('mkl')
  PACKAGES+=('nose')
  PACKAGES+=('numba')
  PACKAGES+=('numexpr')
  PACKAGES+=('numpy')
  PACKAGES+=('pandas')
  PACKAGES+=('pillow')
  PACKAGES+=('pip')
  PACKAGES+=('scikit-image')
  PACKAGES+=('scikit-learn')
  PACKAGES+=('scipy')
  PACKAGES+=('setuptools')
  PACKAGES+=('six')
  PACKAGES+=('sphinx')
  PACKAGES+=('sqlalchemy')
  PACKAGES+=('toolz')
  PACKAGES+=('boost')
  PACKAGES+=('cmake')
  PACKAGES+=('coverage')
  PACKAGES+=('ffmpeg')
  PACKAGES+=('opencv')
  PACKAGES+=('pkg-config')
  PACKAGES+=('protobuf')
  PACKAGES+=('sqlite')
  PACKAGES+=('tensorflow')
  PACKAGES+=('tqdm')
  PACKAGES+=('virtualenv')
  PACKAGES+=('xz')
  PACKAGES+=('zlib')
fi

if [ ! -x ${CONDA} ]; then
  echo "Cannot find conda binary at \`${CONDA}'"
  echo "Update this script to point to a conda installation"
  exit 1
fi

for p in "${PACKAGES[@]}"; do
  /opt/miniconda/bin/conda search $p | grep -e 'h[a-f0-9]\{7\}_[0-9]\{1,3\}' | tail -n 1 | awk '{ print $1, "\b\:\n  \-", $2, " #", $3 }';
done
